﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Tree2
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            treeView1.ShowNodeToolTips = true;
            treeView1.Nodes.Clear();
            DriveInfo[] disks = DriveInfo.GetDrives();
            foreach (DriveInfo disk in disks)
                display(treeView1.Nodes, disk.RootDirectory);
        }

        private void display (TreeNodeCollection branch,
                              DirectoryInfo dir,
                              int level = 2)
        {
            MyNode node = new MyNode ();
            node.Text = dir.Name;
            node.ToolTipText = dir.FullName;
            node.path = dir.FullName;
            branch.Add (node);

            if (level > 1)
               update (node, dir, level);
        }

        private void update (MyNode node,
                             DirectoryInfo dir,
                             int level)
        {
            node.ready = true;
            try
            {
                    DirectoryInfo[] dirs = dir.GetDirectories();
                    foreach (DirectoryInfo d in dirs)
                        display (node.Nodes, d, level-1);
             }
             catch (Exception ex)
             {
                 TreeNode info = new TreeNode();
                 info.Text = ex.ToString();
                 info.ForeColor = Color.Red;
                 node.Nodes.Add (info);
             }
        }

        private void button1_Click(object sender, EventArgs e)
        {

            /*
            TreeNode item = new TreeNode();
            item.Text = "Branch";
            treeView1.Nodes.Add (item);

            TreeNode node = new TreeNode();
            node.Text = "Sub-item";
            item.Nodes.Add (node);
            */

            /*
            ListViewItem item = new ListViewItem ();
            item.Text = "New";
            item.BackColor = Color.Blue;
            item.ForeColor = Color.Red;
            listView1.Items.Add (item);
            item.SubItems.Add (new ListViewItem.ListViewSubItem ());
            item.SubItems.Add ("456");
            */

            /*
            // using System.IO;
            // DirectoryInfo dir = new DirectoryInfo(".");
            var dir = new DirectoryInfo (".");
            FileInfo [] files = dir.GetFiles ();
            // var files = dir.GetFiles ();
            // for (int i = 0; i < files.Length; i++) { FileInfo f = files[i]; }
            listView1.Items.Clear ();
            listView1.ShowItemToolTips = true;
            foreach (var f in files)
            {
               var item = new ListViewItem ();
               item.Text = f.Name;
               item.ToolTipText = f.FullName;
               item.SubItems.Add (f.Length.ToString ("### ### ##0"));
               item.SubItems.Add (f.LastWriteTime.ToString ("yyyy-MM-dd HH:mm:ss.ffff"));
               listView1.Items.Add(item);
            }
            */
        }

        private void treeView1_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            MyNode node = e.Node as MyNode;
            if (node != null)
                foreach (TreeNode t in node.Nodes)
                {
                    MyNode n = t as MyNode;
                    if (n != null && !n.ready)
                        update (n, new DirectoryInfo(n.path), 2);
                }
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            MyNode node = e.Node as MyNode;
            listView1.Items.Clear ();
            listView1.ShowItemToolTips = true;
            toolStripStatusLabel1.Text = "";
            if (node != null)
            {
                try
                {
                    var dir = new DirectoryInfo (node.path);
                    var files = dir.GetFiles();
                    foreach (var f in files)
                    {
                        var item = new ListViewItem();
                        item.Text = f.Name;
                        item.ToolTipText = f.FullName;
                        item.Tag = f.FullName; 
                        item.SubItems.Add (f.Length.ToString("### ### ##0"));
                        item.SubItems.Add (f.LastWriteTime.ToString("yyyy-MM-dd HH:mm:ss.ffff"));
                        listView1.Items.Add (item);
                    }
                }
                catch (Exception ex)
                {
                    String[] lines = ex.Message.Split ('\r');
                    toolStripStatusLabel1.Text = lines[0];
                    // toolStripStatusLabel1.Text = ex.Message.Replace("\r", "-").Replace("\n", "-");
                }
            }
        }

        private void orientationToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void verticalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (verticalToolStripMenuItem.Checked)
                splitContainer2.Orientation = Orientation.Horizontal;
            else
                splitContainer2.Orientation = Orientation.Vertical;

            verticalToolStripMenuItem.Checked = !verticalToolStripMenuItem.Checked;
        }

        private void listView1_ItemActivate(object sender, EventArgs e)
        {
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                String fileName = item.Tag as String;
                if (fileName != null)
                {
                    FileInfo f = new FileInfo (fileName);
                    Text = f.Extension;
                    switch (f.Extension)
                    {
                        case ".jpeg" :
                        case ".jpg" :
                        case ".bmp":
                        case ".png":
                            pictureBox1.Image = new Bitmap(fileName);
                            tabControl1.SelectedTab = imageTab;
                            break;
                        case ".txt":
                            richTextBox1.LoadFile(fileName, RichTextBoxStreamType.PlainText);
                            tabControl1.SelectedTab = textTab;
                            break;
                        case ".rtf":
                            richTextBox1.LoadFile (fileName);
                            tabControl1.SelectedTab = textTab;
                            break;
                        case ".htm":
                        case ".html":
                            webBrowser1.Url = new Uri ("file:/" + fileName);
                            tabControl1.SelectedTab = htmlTab;
                            break;
                        case ".pas":
                        case ".c":
                        case ".cpp":
                            FileStream stream = File.Open (fileName, FileMode.Open);
                            String s = "";
                            bool done = false;
                            while (! done)
                            {
                               byte [] a = new byte [1024];
                               int ans = stream.Read (a, 0, a.Length);
                               foreach (byte b in a)
                               {
                                  char c = (char) b;
                                  s = s + c;
                               }
                               done = (ans == 0);
                            }
                            richTextBox1.Text = s;
                            tabControl1.SelectedTab = textTab;
                            break;

                    }
                }
            }
        }
    }
}
